require 'minitest/autorun'
require 'minitest/unit'
require 'shell_quotes'

#-- |String| :: CLASS
#++

class ShellQuotesTest < Minitest::Unit::TestCase
	def test_bourne_escape_file_path
		#--	NEGATIVE :: ESCAPE & UNESCAPE
		#	++
		assert_equal kraftwerk = String.new('/home/me/Music/Kraftwerk'),
			kraftwerk.bourne_escape_file_path!\
				.bourne_unescape_file_path!
		#-- POSITIVE :: ESCAPE & UNESCAPE
		#++
		assert_equal album =
			String.new("/home/me/Music/Miles Davis/Bitches' Brew"),
			album.bourne_escape_file_path!\
				.bourne_unescape_file_path!
		#-- RUN
		#++
		assert_equal true, system('ls ' +\
			String.new('/usr/bin/env').bourne_escape_file_path!)
		#-- EXCEPTION
		#++
		assert_raises(ShellQuotes::SQError)\
			{ String.new("/home/me/Music/Miles\ Davis\u{0000}")\
				.bourne_escape_file_path! }
		album = String.new("/home/me/Music/Miles Davis/Bitches' Brew/")
		assert_equal\
			String.new("/home/me/Music/Miles\\ Davis/Bitches\\'\\ Brew/"),
			cloned = album.bourne_escape_file_path
		assert(album != cloned)
	end
# 	
# 	def test_bourne_escape_file_path_null_character_will_raise
#			#-- !! SEE LINKED
#			#	http://stackoverflow.com/questions/15974102/error-message-test-on-minitest
#			#++
# 		exception = proc {
# 			("/home/me/Music/Miles\ Davis\u{0000}").bourne_escape_file_path!
# 			}.must_raise(ShellQuotes::SQError)
# 		exception.message.must_equal(
# 			'The NULL character is disallowed inside a Unix file path.')
# 	end
#--
# 
#		#-- RUN
#		#++
# 	def test_run_bourne_unescape_file_path
# 		crappyName = Dir::home + '/Crap Directory/'
# 		crappyName.bourne_escape_file_path!.bourne_unescape_file_path!
# 		assert_equal 0, Dir::mkdir(crappyName)
# 		Dir::rmdir(crappyName)
# 	end
#++
	
	def test_bourne_escape_arg
		#-- NEGATIVE
		#++
		assert_equal everything = '42', everything.bourne_escape_arg!
		#-- POSITIVE, RUN
		#++
		math = '6 * 7'
		assert_equal String.new('6 \* 7'), math.bourne_escape_arg!
		assert_equal true, system('expr ' + math)
	end
	
	def test_bourne_single_quoted
		#-- NEGATIVE
		#++
		assert_equal String.new("'Hello World'"),
			String.new('Hello World').bourne_single_quoted!
		#-- POSITIVE
		#++
		assert_equal "'It_s the backspace character: \\\\.'",
			("It's the backspace character: \\.").bourne_single_quoted!
		assert_equal "'It*s the backspace character: \\\\.'",
			("It's the backspace character: \\.")\
				.bourne_single_quoted!(replacement = '*')
		#--	RUN
		#++
		cmd = 'echo ' +\
			String.new("It's the backspace character: \\.").bourne_single_quoted!
		assert_equal true, system(cmd)
	end

	def test_bourne_double_quoted
		#-- NEGATIVE
		#++
		assert_equal unchanged = ('"Hello World"'),
			unchanged.bourne_double_quoted!
		#--	POSITIVE
		#++
		assert_equal '"It\`s really quite cheap at \$19.95!"',
			("It\`s really quite cheap at $19.95!").bourne_double_quoted!
		assert_equal '"$me in my $HOME."',
			("$me in my $HOME.").bourne_double_quoted!(variable = 'me')
		assert_equal '"$you and $me."',
			("$you and $me.").bourne_double_quoted!(variable = ['me', '$you'])
		assert_equal(
			'"TOAST :: `ps- acx | grep \'iTunes\' | awk \'{ print $1 }\'`"',
			("TOAST :: `ps- acx | grep 'iTunes' | awk '{ print $1 }'`")\
				.bourne_double_quoted!)
		assert_equal '"YOU\'RE TOAST :: `kill \`ps- acx | grep \'iTunes\' | awk \'{ print $1 }\'\``"',
			("YOU'RE TOAST :: `kill \`ps- acx | grep 'iTunes' | awk '{ print $1 }'\``").bourne_double_quoted!
		#-- RUN
		cmd = 'echo ' +\
			String.new('Hi! Hello.').bourne_double_quoted!
		assert_equal true, system(cmd)
		#++
		#-- EXCEPTION
		#++
		assert_raises(ShellQuotes::SQError)\
			{ String.new("TOAST ::").bourne_double_quoted!(variable = 'me_13') }
		assert_raises(ShellQuotes::SQError)\
			{ String.new("TOAST ::")\
				.bourne_double_quoted!(variable = ['okay', 'me_13']) }
		#--	::	I HAVE NO IDEA HOW TO MAKE THIS WORK
		#	SEE	:: http://stackoverflow.com/questions/25233096/upgraded-to-minitest-5-4-0-now-must-and-wont-doesnt-work-within-minitestt
		#--
# 		error = ->{ String.new(":: TOAST")\
# 			.bourne_double_quoted!(variable = '13;') }\
# 			.must_raise ShellQuotes::SQError
# 		error.message.must_match "Illegal character(s) in argument."
	end

	def test_bourne_escape_substitute
		assert_equal "`ps -acx | grep 'iTunes' | awk '{ print $1 }'`",
		String.new("ps -acx | grep 'iTunes' | awk '{ print $1 }'")\
			.bourne_escape_substitute!
		assert_equal(
			"`kill \\`ps -acx | grep 'iTunes' | awk '{ print $1 }'\\``",
		String.new("kill `ps -acx | grep 'iTunes' | awk '{ print $1 }'`")\
			.bourne_escape_substitute!)
	end

end