require 'minitest/autorun'
require 'minitest/unit'
require 'shell_quotes'

#-- |String| :: CLASS
#++

class ShellQuotesTest < Minitest::Unit::TestCase
	def test_csh_escape_file_path
		#--	NEGATIVE
		#	++
		assert_equal (unchanged = '/home/me/Music/Kraftwerk/'),
			unchanged.csh_escape_file_path!
		#-- POSITIVE
		#++
		assert_equal\
			String.new("/home/me/Music/Miles\\ Davis/Bitches\\'\\ Brew/"),
			String.new("/home/me/Music/Miles Davis/Bitches' Brew/")\
				.csh_escape_file_path!
		#-- RUN
		#++
		assert_equal true, system('ls ' +\
			String.new('/usr/bin/env').csh_escape_file_path!)
		#-- EXCEPTION
		#++
		assert_raises(ShellQuotes::SQError)\
			{ String.new("/home/me/Music/Miles\ Davis\u{0000}")\
				.csh_escape_file_path! }
		album = String.new("/home/me/Music/Miles Davis/Bitches' Brew/")
		assert_equal\
			String.new("/home/me/Music/Miles\\ Davis/Bitches\\'\\ Brew/"),
			cloned = album.csh_escape_file_path
		assert(album != cloned)
	end
# 	
# 	def test_csh_escape_file_path_null_character_will_raise
#			#-- !! SEE LINKED
#			#	http://stackoverflow.com/questions/15974102/error-message-test-on-minitest
#			#++
# 		exception = proc {
# 			("/home/me/Music/Miles\ Davis\u{0000}").csh_escape_file_path!
# 			}.must_raise(ShellQuotes::SQError)
# 		exception.message.must_equal(
# 			'The NULL character is disallowed inside a Unix file path.')
# 	end
	
	def test_csh_unescape_file_path
		#--	NEGATIVE
		#--
		assert_equal unchanged = '/home/me/Music/Kraftwerk/',
			unchanged.csh_unescape_file_path!
		#--	POSITIVE
		#++
		assert_equal\
			String.new("/home/me/Music/Miles Davis/Bitches' Brew/"),
			String.new("/home/me/Music/Miles\\ Davis/Bitches\\'\\ Brew/")\
				.csh_unescape_file_path!
	end
#--
# 
#		#-- RUN
#		#++
# 	def test_run_csh_unescape_file_path
# 		crappyName = Dir::home + '/Crap Directory/'
# 		crappyName.csh_escape_file_path!.csh_unescape_file_path!
# 		assert_equal 0, Dir::mkdir(crappyName)
# 		Dir::rmdir(crappyName)
# 	end
#++
	
	def test_csh_escape_arg
		#-- NEGATIVE
		#++
		assert_equal everything = '42', everything.csh_escape_arg!
		#-- POSITIVE, RUN
		#++
		math = '6 * 7'
		assert_equal String.new('6 \* 7'), math.csh_escape_arg!
		assert_equal true, system('expr ' + math)
	end
	
	def test_csh_single_quoted
		#-- NEGATIVE
		#++
		assert_equal String.new("'Hello World'"),
			String.new('Hello World').csh_single_quoted!
		#-- POSITIVE
		#++
		assert_equal "'It_s the economy, stupid\\!'",
			("It's the economy, stupid!").csh_single_quoted!
		assert_equal "'It*s the economy, stupid\\!'",
			("It's the economy, stupid!")\
				.csh_single_quoted!(replacement = '*')
		#--	RUN
		#++
		cmd = 'echo ' +\
			String.new("It's the backspace character: \\.").csh_single_quoted!
		assert_equal true, system(cmd)
	end

	def test_csh_double_quoted
		#-- NEGATIVE
		#++
		assert_equal unchanged = ('"Hello World"'),
			unchanged.csh_double_quoted!
		#--	POSITIVE
		#++
		assert_equal '"It\`s really quite cheap at \$19.95!"',
			("It\`s really quite cheap at $19.95!").bourne_double_quoted!
		assert_equal '"$me in my $HOME."',
			("$me in my $HOME.").csh_double_quoted!(variable = 'me')
		assert_equal '"$you and $me."',
			("$you and $me.").csh_double_quoted!(variable = ['me', '$you'])
		assert_equal(
			'"TOAST :: `ps- acx | grep \'iTunes\' | awk \'{ print $1 }\'`"',
			("TOAST :: `ps- acx | grep 'iTunes' | awk '{ print $1 }'`")\
				.csh_double_quoted!)
		#-- RUN
		cmd = 'echo ' +\
			String.new('Hi! Hello.').bourne_double_quoted!
		assert_equal true, system(cmd)
		#++
		#-- EXCEPTION
		#++
		assert_raises(ShellQuotes::SQError)\
			{ String.new("TOAST ::").csh_double_quoted!(variable = 'me_13') }
		assert_raises(ShellQuotes::SQError)\
			{ String.new("TOAST ::")\
				.csh_double_quoted!(variable = ['okay', 'me_13']) }
	end

	def test_csh_escape_substitute
		assert_equal "`ps -acx | grep 'iTunes' | awk '{ print $1 }'`",
		String.new("ps -acx | grep 'iTunes' | awk '{ print $1 }'")\
			.csh_escape_substitute!
	end

end