require 'minitest/autorun'
require 'minitest/unit'
require 'shell_quotes'

#--	|Shellquotes| :: MODULE
#++

class ShellQuotesTest < Minitest::Unit::TestCase
	def test_shell_quotes_escape
		#-- NEGATIVE
		#++
		str = String.new("/home/me/Music/")
		assert_equal (str + ' '),
			ShellQuotes.bourne_escape(:path => str)
		assert_equal (str + ' '),
			ShellQuotes.bash_escape(:path => str)
		assert_equal (str + ' '),
			ShellQuotes.csh_escape(:path => str)
		#-- POSITIVE
		#++
		assert_equal\
			String.new("/home/me/Music/Miles\\ Davis/Bitches\\'\\ Brew/ "),
			ShellQuotes.bourne_escape(
				:path => String.new("/home/me/Music/Miles Davis/Bitches' Brew/"))
		assert_equal String.new(
"~/Music/Serge\\ Gainsbourg/Histoire\\ de\\ Melody\\ Nelson/04\\ Ah\\!\\ Melody.m4a "
			),
			ShellQuotes.bash_escape(:path => String.new(
"~/Music/Serge Gainsbourg/Histoire de Melody Nelson/04 Ah! Melody.m4a"
			))
		assert_equal String.new(
"~/Music/Serge\\ Gainsbourg/Histoire\\ de\\ Melody\\ Nelson/04\\ Ah\\!\\ Melody.m4a "
			),
			ShellQuotes.csh_escape(:path => String.new(
"~/Music/Serge Gainsbourg/Histoire de Melody Nelson/04 Ah! Melody.m4a"
			))
		assert_equal\
		"ls  /home/me/Music/Thelonius\\ Monk/ /home/me/Music/Miles\\ Davis/ ",
		ShellQuotes.bourne_escape(
			{ :cmd => "ls ", :path => "/home/me/Music/Thelonius Monk/" },
			{ :path => "/home/me/Music/Miles Davis/" })
		home = Dir.new Dir.home
		assert_equal "ls #{home.path.bourne_escape_file_path} ",
			ShellQuotes.bourne_escape(:cmd => 'ls', :path => home.path)
		assert_equal String.new("'It_s for real this time\\!' "),
			ShellQuotes.csh_escape(
				:single_quote => String.new("It's for real this time!"))
		assert_equal String.new("'It*s for real this time\\!' "),
			ShellQuotes.csh_escape(
				:single_quote => String.new("It's for real this time!"),
				:replace => '*')
		str = String.new('$me, this is my $UID')
		assert_equal '"' + str + '" ',
			ShellQuotes.bourne_escape(:double_quote => str, :variable => "me")
		str = String.new('$me, $here I am.')
		assert_equal '"' + str + '" ',
			ShellQuotes.bourne_escape(:double_quote => str,
				:variable => ['$here' , 'me'])
		#-- EXCEPTION
		#++
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(:fluffy => 'bunnies') }
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(:pass => Integer) }
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(:path => Integer) }
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(:variable => Integer) }
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(
				{ :double_quote => str }, { :variable => "me" }) }
		assert_raises(ShellQuotes::SQError)\
			{ ShellQuotes.bourne_escape(
				{ :double_quote => "It's me!" }, { :replace=> "*" }) }
	end
	
	#-- |Dir| :: CLASS
	#++
	def test_dir_bourne_escaped_file_path
		home = Dir.new Dir.home
		assert_equal home.bourne_escaped_file_path,
			Dir.new(home).bourne_escaped_file_path
	end
	
	def test_dir_list_entries
		usr = Dir.new('/usr/')
		assert_equal "/usr/bin", usr.bourne_escaped_entries(:grep => '^bin')[0]
		assert_equal "/usr/bin",
			usr.bourne_escaped_entries(:grep => %r/^bin/)[0]
	end

end